******************************************************
*  Program...........: Samples.prg
*  Author............: Lauren Clarke Randy Pearson
*  Project...........: Parsing Power Tools
*  Created...........: 04/10/2004
*  Copyright.........:
*  Description.......: Code samples for "Parsing Power Tools"
*  Unit Test.........: None
*  Dependencies......: VFP8, RELATION
*  Assumptions.......: SET EXACT OFF
*  Last Change.......: 05/20/2004
*  Change Log........: See end of file
******************************************************
* COMMENTS
*
* Code samples for "Parsing Power Tools Series" by Lauren Clarke and Randy Pearson
* search for the code by Code Listing Number.
* Samples have been gathered into functions for easy calling.
*

* Comment/Uncomment the lines below and execute this prg to see each secton's code samples
CLEAR
* PartOneSamples()
PartTwoSamples()

***********************************************************************************************
** PART ONE SAMPLES
***********************************************************************************************

FUNCTION PartOneSamples()

  ? [==== Results: Code Listing 1 ====]
  lcList =  [205.180.85.40;205.140.111.203;206.63.99.226;]+;
    [206.63.99.227;206.63.99.228]
  lnEndPos   = 0
  lnStartPos = 0
  FOR lnK = 1 TO OCCURS([;], lcList) + 1
    lnEndPos = AT( [;], lcList, lnK )
    lnEndPos = IIF( lnEndPos = 0, LEN( lcList ) + 1, lnEndPos )
    lcIP     = SUBSTR( lcList, lnStartPos+1, lnEndPos-lnStartPos-1 )
    ? lcIP   && <= print the IP
    lnStartPos = AT( [;], lcList, lnK )
  ENDFOR


  ? [==== Results: Code Listing 2 ====]

  lcList =  [205.180.85.40;205.140.111.203;206.63.99.226;]+;
    [206.63.99.227;206.63.99.228]

  ALINES(laIPs, lcList, .T., [;] )

  FOR EACH lcIP IN laIPs
    ? m.lcIP
  ENDFOR

  ? [==== Results: Code Listing 3 ====]
  lcList =  [205.180.85.40;205.140.111.203;206.63.99.226;]+;
    [206.63.99.227;206.63.99.228]
  FOR lnK = 1 TO GETWORDCOUNT( lcList, [;] )
    ? GETWORDNUM( lcList, lnK, [;] )
  ENDFOR

  ? [==== Results: Code Listing 4 ====]
  lcList = [205.180.85.40,205.140.111.203;206.63.99.226;206.63.99.227;206.63.99.228;192.168.0.1]
  FOR lnK = 1 TO GETWORDCOUNT( lcList, [;,] )
    ? GETWORDNUM( lcList, lnK, [;,] )
  ENDFOR

  ? [==== Results: Code Listing 5 ====]
*-- normally, we'd extract the colorscheme string from the ini
*   with an API call or another parsing routine
  lcColorscheme=[background,red|foreground,blue|fontcolor,green]

*-- extract the color scheme data from the .ini into an object
  loCScheme = CREATEOBJECT("empty")
  FOR lnK = 1 TO GETWORDCOUNT( lcColorscheme, [|] )
    lcPropValue = GETWORDNUM( lcColorscheme, lnK , [|] )
    lcProperty = GETWORDNUM( lcPropValue,1,[,] )
    lcValue = GETWORDNUM( lcPropValue,2,[,] )
    ADDPROPERTY( loCScheme,lcProperty,lcValue )
  ENDFOR

  WITH loCScheme
    ? .background  && prints "red"
    ? .foreground  && prints "blue"
    ? .fontcolor   && prints "green"
  ENDWITH

  ? [==== Results: Code Listing 6 ====]

*-- see below for UDF
  ? DecimalPlaces(1234.5678)   && prints 4
  ? DecimalPlaces($1234.5678)  && prints 4
  ? DecimalPlaces("1234.5678") && prints 4


  ? [==== Results: Code Listing 7 ====]
TEXT to lcText textmerge noshow pretext 3
    This is a test,
    note how the initial spaces are trimmed
    this is due to the pretext 1 directive above.
    Today is: <<date()>>
ENDTEXT


  ? lcText


  lcText = [The date is <<date()>>]
  ? TEXTMERGE( lcText )
*-- calls to textmerge can be nested
  lcText2 = [The time is <<time()>>, and <<textmerge(lcText)>>]
  ? TEXTMERGE( lcText2 )
*-- or, can be recursed internally
  lcText2 = [The time is <<time()>>, and <<lcText>>]
  ? TEXTMERGE( lcText2 , .T. )


ENDFUNC


*-----------------------------------------------------*
FUNCTION inListString( tcItem, tcList, tcDelimiter, tltrim )

  LOCAL lcTrimmedList
  tcDelimiter = IIF( VARTYPE( m.tcDelimiter ) # [C], [,] , m.tcDelimiter )

  IF m.tltrim
    tcItem = ALLTRIM( m.tcItem )
*-- strip all leading and trailing spaces from list
    lcTrimmedList = []
    FOR lnK = 1 TO GETWORDCOUNT( m.tcList, m.tcDelimiter )
      lcTrimmedList = m.lcTrimmedList + GETWORDNUM( m.tcList, m.lnK, m.tcDelimiter+[ ] )+m.tcDelimiter
    ENDFOR
    m.tcList = m.lcTrimmedList
  ENDIF

  tcItem = m.tcDelimiter + m.tcItem + m.tcDelimiter
  tcList = m.tcDelimiter + m.tcList + m.tcDelimiter

  RETURN m.tcItem $ m.tcList
ENDFUNC



*-----------------------------------------------------*
FUNCTION DecimalPlaces( tnValue ) AS INTEGER
* returns the number of decimal places present in tnValue
  LOCAL lnSetPoint, lnRetVal, lcStringVal
  lcStringVal = PADR( tnValue, 40 )   && max 18 either side of .
  lnRetVal    = 0
  lnSetPoint  = AT( SET("point"), m.lcStringVal )
  IF lnSetPoint > 0
    lnRetVal = LEN( RTRIM( m.lcStringVal ) ) - m.lnSetPoint
  ENDIF
  RETURN m.lnRetVal
ENDFUNC

***********************************************************************************************
** PART TWO SAMPLES
***********************************************************************************************

FUNCTION PartTwoSamples()

  LOCAL lcDemos, lcDecimalToCommaTest, laDemos(1)

TEXT to lcDecimalToCommaTest noshow
    A pair of shoes costs 12.85, an ice-cream cone is 2.60 and a sandwich is 123,412.97.
ENDTEXT

TEXT TO lcDemos NOSHOW pretext 7
    * --- Results: Code Listing from Part Two ---
    validLP("abc123")
    validLP("a1c2b3")

    validLPregExp("abc123")
    validLPregExp("a1c2b3")

    decimalToComma1(lcDecimalToCommaTest)
    decimalToComma2(lcDecimalToCommaTest)
    decimalToComma3(lcDecimalToCommaTest)

    *-- generate three full conversion files for diff
    *strtofile( decimalToComma1(filetostr([ebay2003.html])),[ebay2003_1.html])
    *strtofile( decimalToComma2(filetostr([ebay2003.html])),[ebay2003_2.html])
    *STRTOFILE( decimalToComma3(FILETOSTR([ebay2003.html])),[ebay2003_3.html])

    StrtranRegExp( [The sandwich. cost $5.75. Nice Sandwich.], [(\d{1,3})\.(\d{1,})],[$1,$2] )
    StrtranRegExp( [The sandwich. cost $123,345.75. Nice Sandwich.], [(\d{1,3}?),(\d{1,3}?)],[$1 $2] )

    *-- count words
    OccursRegExp( "\b(\w+)\b",[the then quick quick brown fox fox])
    *-- count repeated words
    OccursRegExp( "\b(\w+)\s\1\b",[the then quick quick brown fox fox])
ENDTEXT

  ALINES( laDemos, lcDemos )

  FOR EACH lcLine IN laDemos
    ? lcLine, [=], IIF(lcLine=[*] OR EMPTY(lcLine),[],EVAL( lcLine ))
  ENDFOR

ENDFUNC



*----------------------------------------------------------
FUNCTION decimalToComma1( tcText )
*-- VFP, with substr
  LOCAL laLines(1), lcLine, lnLines, lcRet, lcTemp

*-- standardize the digits
  lcTemp = CHRTRAN( m.tcText,[0123456789],REPLICATE([#],10))

*-- deal with the decimal places
  FOR lnJ = 1 TO OCCURS( [#.#], m.lcTemp )
    lnPos = AT( [#.#], m.lcTemp, m.lnJ )
    tcText = SUBSTR( m.tcText,1,m.lnPos )+[,]+SUBSTR( m.tcText,m.lnPos+2 )
  ENDFOR

*-- deal with the commas
  FOR lnJ = 1 TO OCCURS( [#,#], m.lcTemp )
    lnPos = AT( [#,#], m.lcTemp, m.lnJ )
    tcText = SUBSTR( m.tcText,1,m.lnPos )+[ ]+SUBSTR( m.tcText,m.lnPos+2 )
  ENDFOR

  RETURN m.tcText

ENDFUNC


*----------------------------------------------------------
FUNCTION decimalToComma2( tcText )

  LOCAL laLines(1), lcLine, lnLines, lcRet, lcTemp

  lnLines = ALINES(laLines, m.tcText, CHR(13) )

  FOR lnK = 1 TO m.lnLines
    lcLine = laLines(m.lnK)

*-- standardize the digits
    lcTemp = CHRTRAN( m.lcLine,[0123456789],REPLICATE([#],10))

*-- deal with the decimals
    FOR lnJ = 1 TO OCCURS( [#.#], m.lcTemp )
      lnPos = AT( [#.#], m.lcTemp, m.lnJ )
      m.lcLine = SUBSTR( m.lcLine,1,m.lnPos )+[,]+SUBSTR( m.lcLine,m.lnPos+2 )
    ENDFOR

*-- deal with the commas
    FOR lnJ = 1 TO OCCURS( [#,#], m.lcTemp )
      lnPos = AT( [#,#], m.lcTemp, m.lnJ )
      m.lcLine = SUBSTR( m.lcLine,1,m.lnPos )+[ ]+SUBSTR( m.lcLine,m.lnPos+2 )
    ENDFOR

    laLines(m.lnK) = m.lcLine
  ENDFOR
  lcRet = []
  FOR lnK = 1 TO m.lnLines
    lcRet= m.lcRet + laLines( m.lnK ) + CHR(13)
  ENDFOR
  RETURN m.lcRet
ENDFUNC

*----------------------------------------------------------
FUNCTION decimalToComma3( tcText )
*-- Regular expressions approach
  LOCAL loRE
  loRE = CREATEOBJECT("vbscript.regexp")
  WITH loRE
    .GLOBAL     = .T.
    .multiline  = .T.
    .ignorecase = .T.
*-- deal with the commas
    .PATTERN    = [(\d{1,3}?),(\d{1,3}?)]
    tcText      = .REPLACE( m.tcText , [$1 $2])
*-- deal with the decimals
    .PATTERN    = [(\d{1,3})\.(\d{1,})]
    RETURN .REPLACE( m.tcText , [$1,$2])
  ENDWITH
ENDFUNC





*----------------------------------------------------------
FUNCTION validLP( tcString )
*[[ paramter checking suppressed for brevity
  tcString = ALLTRIM( tcString )

*-- can't be valid if length # 6
  IF LEN( tcString ) # 6
    RETURN .F.
  ENDIF

*-- standardize all the digits to '#' symbol
  tcString = CHRTRAN( UPPER( tcString ), [1234567890] , REPLICATE([#],10))
*-- standardize all the characters to 'A'
  tcString = CHRTRAN( tcString, [ABCDEFGHIJKLMNOPQRSTUVWXYZ],REPLICATE([A],26) )

*-- check for valid pattern
  RETURN tcString == [AAA###]
ENDFUNC

*----------------------------------------------------------
FUNCTION validLPregExp( tcString )
*-- same as validLP() above, but uses regular expressions
  loRE = CREATEOBJECT("vbscript.regexp")
  WITH loRE
    .PATTERN = "^[A-Za-z]{3}\d{3}$"
    RETURN .test( tcString )
  ENDWITH
ENDFUNC

*----------------------------------------------------------
FUNCTION validLPregx2( tcString, toRE )
*-- same as validLP() above, but uses passed regular expressions
  WITH toRE
    .PATTERN = "^[A-Za-z]{3}\d{3}$"
    RETURN .test( tcString )
  ENDWITH
ENDFUNC

*----------------------------------------------------------
FUNCTION StrtranRegExp( tcSourceString, tcPattern, tcReplace )
  LOCAL loRE
  loRE = CREATEOBJECT("vbscript.regexp")
  WITH loRE
    .PATTERN = tcPattern
    .GLOBAL  = .T.
    .multiline = .T.
    RETURN .REPLACE( tcSourceString , tcReplace )
  ENDWITH
ENDFUNC

*----------------------------------------------------------
FUNCTION OccursRegExp(tcPattern, tcText)
  LOCAL loRE, loMatches, lnResult
  loRE = CREATEOBJECT("vbscript.regexp")
  WITH loRE
    .PATTERN = m.tcPattern
    .GLOBAL  = .T.
    .multiline = .T.
    loMatches = loRE.Execute( m.tcText )
    lnResult  = loMatches.COUNT
    loMatches = NULL
  ENDWITH
  RETURN m.lnResult
ENDFUNC
