; -- myVFPApp.iss --
; This is the setup script for myVFPApp version 1.0.0.

[Setup]
AppID=MyVFPApp
AppName=MyVFPApp
AppVerName=MyVFPApp version 1.0.0
AppCopyright=Copyright  2004 Information Technology Associates
DefaultDirName={pf}\MyVFPApp
AppPublisher=Information Technology Associates
AppPublisherURL=http://www.ita-software.com
AppSupportURL=http://www.ita-software.com
AppUpdatesURL=http://www.ita-software.com
UsePreviousAppDir=yes
DirExistsWarning=auto
DefaultGroupName=MyVFPApp
SourceDir=C:\FoxTalk\MyVFPApp
OutputBaseFilename=myVFPApp 1.0.0 Setup
OutputDir=C:\FoxTalk\MyVFPApp\Distrib
LicenseFile=license.rtf
MinVersion=4.1.2222,5.0.2195
WizardImageFile=compiler:images\WizModernImage13.bmp
WizardSmallImageFile=compiler:images\WizModernSmallImage13.bmp

[Types]
Name: full; Description: Full installation
Name: workstation; Description: Workstation installation
Name: database; Description: Database installation
Name: custom; Description: Custom installation; Flags: iscustom

[Components]
Name: workstation; Description: Workstation files; Types: full workstation
Name: database; Description: database files; Types: full database

[Registry]
Root: HKCU; Subkey: Software\ITA; Components: workstation; Flags: uninsdeletekeyifempty
Root: HKCU; Subkey: Software\ITA\myVFPApp; Components: workstation; Flags: uninsdeletekey
Root: HKCU; Subkey: Software\ITA\myVFPApp\Settings; ValueType: string; ValueName: AppDir; ValueData: {code:GetAppDir}; Components: workstation
Root: HKCU; Subkey: Software\ITA\myVFPApp\Settings; ValueType: string; ValueName: DataDir; ValueData: {code:GetDataDir}; Components: database

[Tasks]
Name: desktopicon; Description: Create a &desktop icon; GroupDescription: Additional icons:

[Dirs]
Name: {code:GetDataDir}; Components: database; Flags: uninsneveruninstall

[Files]
;**********************************************************************
;
;	Workstation component files
;
;**********************************************************************
Source: MyVFPApp.exe; DestDir: {app}; Components: workstation
Source: Readme.txt; DestDir: {app}; Components: workstation; Flags: isreadme
;**********************************************************************
;
;	Database component files
;
;**********************************************************************
Source: Data\customers.DBF; DestDir: {code:GetDataDir}; Flags: onlyifdoesntexist; Components: database
Source: Data\customers.CDX; DestDir: {code:GetDataDir}; Flags: onlyifdoesntexist; Components: database

;**********************************************************************
;
;        VFP8 runtime support libraries and related files
;
;**********************************************************************
#include "VFP8Runtimes.txt"

[Icons]
Name: {group}\myVFPApp; Filename: {app}\myVFPApp.exe; WorkingDir: {app}; Components: workstation
Name: {group}\Uninstall myVFPApp; Filename: {uninstallexe}; Components: workstation
Name: {userdesktop}\myVFPApp; Filename: {app}\myVFPApp.exe; WorkingDir: {app}; Tasks: desktopicon; Components: workstation

[Run]
Filename: {app}\myVFPApp.exe; Description: Launch myVFPApp; Flags: nowait postinstall skipifsilent

[Code]
var
  DataDir: String;

function InitializeSetup(): Boolean;
begin
  DataDir := GetPreviousData('DataDir', '');
  { Let Setup run }
  Result := True;
end;

procedure RegisterPreviousData(PreviousDataKey: Integer);
begin
{ Store the settings so we can restore them next time }
  SetPreviousData(PreviousDataKey, 'DataDir', DataDir);
end;

function ScriptDlgPages(CurPage: Integer; BackClicked: Boolean): Boolean;
var
  I, CurSubPage: Integer;
  Next, NextOk: Boolean;
begin
  if pos( 'database', WizardSelectedComponents( false)) > 0 then begin // Database component is selected
	  if (not BackClicked and (CurPage = wpSelectComponents)) or (BackClicked and (CurPage = wpReady)) then begin
		{ Insert a custom wizard page between two non custom pages }
		{ First open the custom wizard page }
		ScriptDlgPageOpen();
		{ Set some captions }
		ScriptDlgPageSetCaption('Select Data Directory');
		ScriptDlgPageSetSubCaption1('Where should the data files be installed?');
		ScriptDlgPageSetSubCaption2('Select the folder where you would like to install the data files.');
		{ Initialize the DataDir if necessary }
		if DataDir = '' then
		   DataDir := ExpandConstant('{app}') + '\Data';
	    { Ask for a dir until the user has entered one or click Back or Cancel }
		Next := InputDir(True, '', DataDir);
		while Next and (DataDir = '') do begin
		  MsgBox(SetupMessage(msgInvalidPath), mbError, MB_OK);
		  Next := InputDir(True, '', DataDir);
		end;
		{ See NextButtonClick and BackButtonClick: return True if the click should be allowed }
		if not BackClicked then
		  Result := Next
		else
		  Result := not Next;
		{ Close the wizard page. Do a FullRestore only if the click (see above) is not allowed }
		ScriptDlgPageClose(not Result);
	  end else begin
		Result := True;
	  end;
	end else begin
      Result := True;
    end;
end;

function NextButtonClick(CurPage: Integer): Boolean;
begin
  Result := ScriptDlgPages(CurPage, False);
end;

function BackButtonClick(CurPage: Integer): Boolean;
begin
  Result := ScriptDlgPages(CurPage, True);
end;

function UpdateReadyMemo(Space, NewLine, MemoUserInfoInfo, MemoDirInfo, MemoTypeInfo, MemoComponentsInfo, MemoGroupInfo, MemoTasksInfo: String): String;
var
  S: String;
begin
  //  { Fill the 'Ready Memo' with the normal settings and the custom settings }
  S := '';
  S := MemoComponentsInfo + NewLine + NewLine;

  { Workstation component is selected }
  if pos( 'workstation', WizardSelectedComponents( false)) > 0 then
    S := S + MemoDirInfo + ' (workstation)' + NewLine;

  if pos( 'database', WizardSelectedComponents( false)) > 0 then // Database component is selected
    if pos( 'workstation', WizardSelectedComponents( false)) > 0 then // Workstation component is selected
      S := S + Space + DataDir + ' (database)' + NewLine
    else begin // Workstation component is not selected
	  S := S + 'Destination Directory:' + NewLine;
      S := S + Space + DataDir + ' (database)' + NewLine;
    end;

  Result := S;
end;

function GetDataDir(S: String): String;
begin
  { Return the selected DataDir }
  Result := DataDir;
end;

function GetAppDir(Default: String): String;
begin
  Result := ExpandConstant('{app}');
end;
